<?php

namespace Oro\Bundle\CalendarBundle\Manager\CalendarEvent;

use Doctrine\Common\Persistence\ManagerRegistry;

use Oro\Bundle\CalendarBundle\Entity\CalendarEvent;

/**
 * Responsible to delete calendar event with additional logic:
 * - Exception of recurring event could be cancelled instead of deleted.
 * - Child events for this case will be cancelled instead of deleted as well.
 * - Relations not supported cascade remove will be removed manually.
 */
class DeleteManager
{
    /**
     * @var ManagerRegistry
     */
    protected $doctrine;

    /**
     * @param ManagerRegistry $doctrine
     */
    public function __construct(ManagerRegistry $doctrine)
    {
        $this->doctrine = $doctrine;
    }

    /**
     * This method is responsible to decide and delete or cancel calendar event.
     *
     * Note: Exception of calendar event could be cancelled instead of being deleted.
     *
     * @param CalendarEvent $calendarEvent Calendar event which should be deleted.
     * @param boolean $allowCancel If TRUE and event is an exception of recurring event it will be cancelled instead.
     */
    public function deleteOrCancel(CalendarEvent $calendarEvent, $allowCancel)
    {
        if ($allowCancel && $calendarEvent->getRecurringEvent()) {
            $this->doCancel($calendarEvent);
        } else {
            $this->doDelete($calendarEvent);
        }
    }

    /**
     * If this is an exception of recurring event, cancel instead of remove.
     *
     * @param CalendarEvent $calendarEvent
     */
    protected function doCancel(CalendarEvent $calendarEvent)
    {
        $calendarEvent->setCancelled(true);

        $parentCalendarEvent = $calendarEvent->getParent() ? : $calendarEvent;
        /** @var CalendarEvent $calendarEvent */
        foreach ($parentCalendarEvent->getChildEvents() as $childCalendarEvent) {
            $childCalendarEvent->setCancelled(true);
        }
    }

    /**
     * Delete calendar event from the persistence.
     *
     * @param CalendarEvent $calendarEvent
     */
    protected function doDelete(CalendarEvent $calendarEvent)
    {
        $manager = $this->doctrine->getManager();
        $manager->remove($calendarEvent);

        $this->deleteAndClearRecurringEventExceptions($calendarEvent);
    }

    /**
     * Recurring event exceptions intentionally doesn't have cascade remove, so this method can be used
     * when collection of recurring event exceptions should be cleared and removed.
     *
     * @param CalendarEvent $event
     */
    public function deleteAndClearRecurringEventExceptions(CalendarEvent $event)
    {
        foreach ($event->getRecurringEventExceptions() as $exception) {
            $this->doctrine->getManager()->remove($exception);
        }
        $event->getRecurringEventExceptions()->clear();

        foreach ($event->getChildEvents() as $childEvent) {
            $this->deleteAndClearRecurringEventExceptions($childEvent);
        }
    }
}
